import React, { Component } from 'react';
import PropTypes from 'prop-types';
import { FormattedMessage } from 'react-intl';
import QRCode from 'qrcode.react';

import Button from 'reactstrap/lib/Button';

import { IconEth } from '../../icons/icon-eth/icon-eth';
import { IconScr } from '../../icons/icon-scr/icon-scr';
import { IconButtonLoader } from '../../icons/icon-button-loader/icon-button-loader';
import { IconDotPreloader } from '../../icons/icon-dot-preloader/icon-dot-preloader';
import { InputWithIcon } from '../../form/input-with-icon/input-with-icon';
import { InputWithClipboard } from '../../form/input-with-clipboard/input-with-clipboard';

import { CHAR_CODES } from '../../../constants';

import './send-amount.scss';

export class SendAmount extends Component {
  static propTypes = {
    rate: PropTypes.string,
    ethValue: PropTypes.string,
    scrValue: PropTypes.string,
    maxAmount: PropTypes.string,
    isLoading: PropTypes.bool,
    ethAddress: PropTypes.string,
    onEthChange: PropTypes.func.isRequired,
    onScrChange: PropTypes.func.isRequired,
    onDoneClick: PropTypes.func.isRequired,
    isEthLoading: PropTypes.bool,
    isScrLoading: PropTypes.bool,
    isMaxAmountLoading: PropTypes.bool,
  };

  static defaultProps = {
    rate: null,
    ethValue: '',
    scrValue: '',
    isLoading: false,
    maxAmount: null,
    ethAddress: null,
    isEthLoading: false,
    isScrLoading: false,
    isMaxAmountLoading: false,
  };

  checkValidAmountValue = amount => /^\d+.?\d{0,9}$/.test(amount) && !/^0\d/.test(amount);

  onKeyPress = (evt) => {
    const { which, keyCode } = evt;
    const charCode = which || keyCode;
    if ((charCode > CHAR_CODES.CONTROL &&
      ((charCode < CHAR_CODES.ZERO && charCode !== CHAR_CODES.DOT)
      || charCode > CHAR_CODES.NINE))
    ) {
      evt.preventDefault();
    }
  }

  ethChangeHandler = (e) => {
    const { value } = e.target;

    if (value.length && !this.checkValidAmountValue(value)) {
      return;
    }

    this.props.onEthChange(value);
  }

  scrChangeHandler = (e) => {
    const { value } = e.target;

    if (value.length && !this.checkValidAmountValue(value)) {
      return;
    }

    this.props.onScrChange(value);
  }

  render() {
    const {
      ethValue, scrValue, rate, maxAmount, onDoneClick, ethAddress, isLoading, isEthLoading,
      isScrLoading, isMaxAmountLoading,
    } = this.props;

    return (
      <div className="send-amount-step">
        <h3 className="text-center mb-4">
          <FormattedMessage id="universal.balance.refill.send-step.title" />
        </h3>

        <div className="qr mx-auto mb-4 border border-shade-s30 rounded position-relative">
          {isLoading ? <IconButtonLoader withAbsolutePosition /> : <QRCode className="m-1_5" value={ethAddress} />}
        </div>

        <InputWithClipboard value={ethAddress} isLoading={isLoading} />

        <div className="divider my-3" />

        <div className="d-flex align-items-center mb-4">
          <InputWithIcon
            icon={<IconEth />}
            value={ethValue}
            onChange={this.ethChangeHandler}
            className="pr-5"
            isLoading={isEthLoading}
            onKeyPress={this.onKeyPress}
          />
          <h5 className="mx-1_5 text-gray-dark-hint">=</h5>
          <InputWithIcon
            icon={<IconScr />}
            value={scrValue}
            onChange={this.scrChangeHandler}
            className="pr-5"
            isLoading={isScrLoading}
            onKeyPress={this.onKeyPress}
          />
        </div>

        <div className="d-flex justify-content-between">
          <span className="text-gray-dark-hint">
            <FormattedMessage id="universal.balance.refill.calc-step.rate" />
          </span>
          {!rate && (isEthLoading || isScrLoading) ? <IconDotPreloader className="h5p dot-preloader-small" />
            : (
              <span className="h5p ml-1 text-nowrap">
                1 SCR = {rate} ETH
              </span>
          )}
        </div>

        <div className="divider my-2" />

        <div className="d-flex justify-content-between">
          <span className="text-gray-dark-hint">
            <FormattedMessage id="universal.balance.refill.calc-step.max.title" />
          </span>
          {isMaxAmountLoading ? <IconDotPreloader className="h5p dot-preloader-small" />
          : (
            <span className="h5p ml-1 text-nowrap">
              <FormattedMessage id="universal.balance.refill.calc-step.max.value" values={{ value: maxAmount }} />
            </span>
          )}
        </div>

        <Button outline block className="mt-4" onClick={onDoneClick}>
          <FormattedMessage id="universal.done" />
        </Button>
      </div>
    );
  }
}
